package com.njyunzhi.invoice.common.excel;

import com.alibaba.excel.metadata.Head;
import com.alibaba.excel.write.merge.AbstractMergeStrategy;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.util.CellRangeAddress;

import java.util.ArrayList;
import java.util.List;

public class MergeRowStrategy extends AbstractMergeStrategy {

    // 对比数据源
    List<Object> dataList;

    // 从第几行开始合并
    Integer rowStart;

    // 第几列 从 0 开始
    Integer column;

    // 已经处理到第几行
    int processedRow;

    public MergeRowStrategy(List<Object> dataList, Integer rowStart, Integer column) {
        this.dataList = dataList;
        this.rowStart = null == rowStart || rowStart < 0 ? 0 : rowStart;
        this.column = null == column || column < 0 ? 0 : column;
    }

    @Override
    protected void merge(Sheet sheet, Cell cell, Head head, Integer relativeRowIndex) {
        CellRangeAddress cellRange = getCellRange(sheet, cell);
        if (null != cellRange) {
            sheet.addMergedRegionUnsafe(cellRange);
        }
    }

    private CellRangeAddress getCellRange(Sheet sheet, Cell cell) {
        int rowIndex = cell.getRowIndex();
        int columnIndex = cell.getColumnIndex();

        // 如果不是需要处理的列, 不处理
        if (column != columnIndex) return null;
        // 如果当前行小于起始数据合并行, 不处理
        if (rowStart > rowIndex) return null;
        // 如果当前行已经处理过, 不处理
        if (processedRow >= rowIndex) return null;

        // 从当前行查询数据, 一直查询相同内容行
        // excel 数据行与 datalist 数据行, index 差值就是 rowStart
        int dataIndex = rowIndex - rowStart;
        int sameRow = 0;
        Object val = dataList.get(dataIndex); // 当前行数据
        for (int i = dataIndex; i < dataList.size(); i += 1) {
            if (i + 1 >= dataList.size()) break;

            Object nextVal = dataList.get(i + 1);
            boolean found = null == val && nextVal == null || (val != null && val.equals(nextVal));
            if (found) {
                sameRow ++;
//                continue;
            } else {
                break;
            }
        }

        if (sameRow > 0) {
            processedRow = rowIndex + processedRow;
            return new CellRangeAddress(rowIndex, processedRow, column, column);
        }

        return null;
    }
}
