package com.njyunzhi.invoice.controller;

import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.njyunzhi.invoice.common.*;
import com.njyunzhi.invoice.entity.SysUser;
import com.njyunzhi.invoice.entity.TaPerson;
import com.njyunzhi.invoice.vo.ChangePassword;
import com.njyunzhi.invoice.vo.LoginParm;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.Map;


@Api(tags ="登录")
@RestController
@RequestMapping("/")
public class LoginController extends BaseController {

    @Value("${yz.mp.openidAPI}")
    String openidAPI;

    @Autowired
    WxMpUtil wxMpUtil;

    @Autowired
    HttpUtils httpUtils;


    @PostMapping("/admin/login")
    @ApiOperation(value="后台登录", notes = "后台登录", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean login(@ApiParam("登录参数") @RequestBody LoginParm loginParm) throws Exception {

        if (StringUtils.isEmpty(loginParm.getUserName()) || StringUtils.isEmpty(loginParm.getPassword())) {
            return ResponseBean.error("登录用户或者密码不能为空");
        }

        SysUser sysUser = iSysUserService.login(loginParm);

        StpUtil.login(sysUser.getUserId(), "admin");
        String tokenValue = StpUtil.getTokenValue();

        Map<String, Object> result = new HashMap<>();
        result.put("user", sysUser);
        result.put("token", tokenValue);

        return ResponseBean.success(result);
    }

    @PostMapping("/admin/logout")
    @ApiOperation(value="登出", notes = "登出", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean logout() throws Exception {
        StpUtil.logout();
        return ResponseBean.success("success");
    }


    @PutMapping("/admin/change-password")
    @ApiOperation(value="修改密码", notes = "修改密码", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean changePassword(@ApiParam("修改密码参数") @RequestBody ChangePassword param) throws Exception {
        if (StringUtils.isEmpty(param.getOriginPassword()) || StringUtils.isEmpty(param.getNewPassword())) {
            return ResponseBean.error("原始密码或新密码不能为空", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        SysUser currentUser = currentUser();

        if (!checkPassword(param.getOriginPassword(), currentUser.getPassword(), currentUser.getUserId())) {
            return ResponseBean.error("原始密码不正确", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        currentUser.setPassword(EncryptUtils.md5(param.getNewPassword(), currentUser.getUserId()));
        iSysUserService.updateById(currentUser);

        return ResponseBean.success("密码修改成功");
    }

    private boolean checkPassword(String src, String targ, String salt) {
        return EncryptUtils.md5(src, salt).equals(targ);
    }



    @PostMapping("/h5/login")
    @ApiOperation(value="H5端登录", notes = "H5端登录", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean h5Login(@ApiParam("登录参数") @RequestParam String code) throws Exception {

        if (StringUtils.isEmpty(code)) {
            return ResponseBean.error("登录 CODE 不存在");
        }

        String openID;
        if (!code.equals("123")) {
            String body = httpUtils.get(String.format(openidAPI, code));
            if (StringUtils.isEmpty(body)) {
                return ResponseBean.error("登录系统失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
            }

            JSONObject jsonObject = JSONUtil.parseObj(body);
            if (jsonObject.getInt("code") != ResponseBean.CODE_SUCCESS) {
                return ResponseBean.error(jsonObject.getStr("message"), ResponseBean.ERROR_UNAVAILABLE);
            }

            openID = jsonObject.getStr("data");
        } else {
            openID = "123";
        }

        TaPerson taPerson = iTaPersonService.getExistBy("openid", openID, false, true);
        if (null == taPerson) {
            taPerson = new TaPerson();
            taPerson.setOpenid(openID);
            taPerson.setStatus(Constants.STATUS_NORMAL);
            if (!iTaPersonService.save(taPerson)) {
                return ResponseBean.error("登录系统失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
            }
        } else {
            if (Constants.STATUS_NORMAL != taPerson.getStatus()) {
                return ResponseBean.error("您的账户异常, 请联系管理人员", ResponseBean.ERROR_UNAVAILABLE);
            }
        }


        StpUtil.login(taPerson.getPersonId(), "h5");
        String tokenValue = StpUtil.getTokenValue();

        Map<String, Object> result = new HashMap<>();
        result.put("person", taPerson);
        result.put("token", tokenValue);

        return ResponseBean.success(result);
    }
}
