package com.njyunzhi.invoice.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.njyunzhi.invoice.common.BaseController;
import com.njyunzhi.invoice.common.ResponseBean;
import com.njyunzhi.invoice.common.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.njyunzhi.invoice.service.ISysOrgService;
import com.njyunzhi.invoice.entity.SysOrg;
import org.springframework.web.bind.annotation.RestController;

/**
 * <p>
    * 单位表 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2022-07-04
 */

@Api(tags = "单位表")
@RestController
@RequestMapping("/")
public class SysOrgController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(SysOrgController.class);

    @Autowired
    public ISysOrgService iSysOrgService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/h5/org",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean sysOrgList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                   @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                   @ApiParam("是否系统内置") @RequestParam(value ="isSystem", required = false) Boolean isSystem,
                                   @ApiParam("单位名称") @RequestParam(value ="name", required = false) String name) throws Exception{

        IPage<SysOrg> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<SysOrg> queryWrapper = new QueryWrapper<>();
        queryWrapper.like(StringUtils.isEmpty(name),"name", "%"+name+"%");
        queryWrapper.eq(isSystem != null,"is_system", isSystem);
        queryWrapper.orderByDesc("create_date");

        IPage<SysOrg> result = iSysOrgService.page(pg, queryWrapper);
        return ResponseBean.success(result);
    }

    /**
     * 保存对象
     * @param sysOrg 实体对象
     * @return
     */
    @RequestMapping(value="/sysOrg",method= RequestMethod.POST)
    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean sysOrgAdd(@ApiParam("保存内容") @RequestBody SysOrg sysOrg) throws Exception{

        if (iSysOrgService.save(sysOrg)){
            return ResponseBean.success(sysOrg);
        }else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @RequestMapping(value="/sysOrg/{id}", method= RequestMethod.DELETE)
    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
    public ResponseBean sysOrgDelete(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{
        if(iSysOrgService.removeById(id)){
            return ResponseBean.success("success");
        }else {
            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param sysOrg 实体对象
     * @return
     */
    @RequestMapping(value="/sysOrg/{id}",method= RequestMethod.PUT)
    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean sysOrgUpdate(@ApiParam("对象ID") @PathVariable Integer id,
                                        @ApiParam("更新内容") @RequestBody SysOrg sysOrg) throws Exception{

        if (iSysOrgService.updateById(sysOrg)){
            return ResponseBean.success(iSysOrgService.getById(id));
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/sysOrg/{id}",method= RequestMethod.GET)
    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean sysOrgGet(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{
        return ResponseBean.success(iSysOrgService.getById(id));
    }
}
