package com.njyunzhi.invoice.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.njyunzhi.invoice.common.*;
import com.njyunzhi.invoice.entity.*;
import com.njyunzhi.invoice.service.ITaInvoiceFillService;
import com.njyunzhi.invoice.service.ITaInvoiceItemService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.njyunzhi.invoice.service.ITaInvoiceDetailService;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
    * 报销明细 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2022-07-04
 */

@Api(tags = "报销明细")
@RestController
@RequestMapping("/")
public class TaInvoiceDetailController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaInvoiceDetailController.class);

    @Autowired
    public ITaInvoiceDetailService iTaInvoiceDetailService;

    @Autowired
    public ITaInvoiceFillService iTaInvoiceFillService;

    @Autowired
    public ITaInvoiceItemService iTaInvoiceItemService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/admin/invoice/{invoiceId}/detail-tpl",method= RequestMethod.GET)
    @ApiOperation(value="模板列表", notes = "模板列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taInvoiceDetailList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
									 @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                     @ApiParam("班级ID") @PathVariable(value ="invoiceId") String invoiceId) throws Exception{

        IPage<TaInvoiceDetail> pg = new Page<>(pageNum, pageSize);
        IPage<TaInvoiceDetail> result = iTaInvoiceDetailService.getTplPageBy(pg, invoiceId);
        return ResponseBean.success(result);
    }


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/admin/invoice/{invoiceId}/detail",method= RequestMethod.GET)
    @ApiOperation(value="报销列表", notes = "报销列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                @ApiParam("班级ID") @PathVariable(value ="invoiceId") String invoiceId,
                                @ApiParam("单位名称") @PathVariable(value ="orgName", required = false) String orgName,
                                @ApiParam("报销人名称") @PathVariable(value ="personName", required = false) String personName) throws Exception{

        IPage<TaInvoiceDetail> pg = new Page<>(pageNum, pageSize);
        IPage<TaInvoiceDetail> result = iTaInvoiceDetailService.getPageBy(pg, invoiceId, orgName, personName);
        return ResponseBean.success(result);
    }

    @RequestMapping(value="/admin/invoice/{invoiceId}/detail-tpl",method= RequestMethod.POST)
    @ApiOperation(value="一键生成", notes = "一键生成", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean taInvoiceDetailAdd(@ApiParam("班级ID") @PathVariable(value ="invoiceId") String invoiceId) throws Exception{

        if (iTaInvoiceDetailService.createBatch(invoiceId)){
            return ResponseBean.success("ok");
        } else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }


    @RequestMapping(value="/admin/invoice/{invoiceId}/detail-tpl/item",method= RequestMethod.POST)
    @ApiOperation(value="添加或者修改项目", notes = "添加或者修改项目", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean createNew(@ApiParam("班级ID") @PathVariable(value ="invoiceId") String invoiceId,
                                  @ApiParam("报销项目") @RequestBody TaInvoiceItem taInvoiceItem) throws Exception {

        if (StringUtils.isEmpty(taInvoiceItem.getInvoicePersonId())) {
            return ResponseBean.error("未设置绑定人员");
        }

        if (StringUtils.isEmpty(taInvoiceItem.getName()) || null == taInvoiceItem.getCharge()) {
            return ResponseBean.error("报销项目名称或者金额不能为空");
        }

        taInvoiceItem.setInvoiceId(invoiceId);
        if (iTaInvoiceDetailService.createOrUpdateItem(taInvoiceItem)){
            return ResponseBean.success("ok");
        } else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    @RequestMapping(value="/admin/invoice/{invoiceId}/detail-tpl/item/{itemId}",method= RequestMethod.DELETE)
    @ApiOperation(value="删除模板项目", notes = "删除模板项目", httpMethod = "DELETE", response = ResponseBean.class)
    public ResponseBean createNew(@ApiParam("班级ID") @PathVariable(value ="invoiceId") String invoiceId,
                                  @ApiParam("报销项目ITEM") @PathVariable(value ="itemId") Integer itemId) throws Exception {
        if (iTaInvoiceDetailService.deleteItem(invoiceId, itemId)){
            return ResponseBean.success("ok");
        } else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    @RequestMapping(value="/h5/invoice/{invoiceId}/detail",method= RequestMethod.PUT)
    @ApiOperation(value="H5确认报销单", notes = "H5确认报销单", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean invoiceConfirm(@ApiParam("班级ID") @PathVariable(value ="invoiceId") String invoiceId,
                                       @ApiParam("更新内容") @RequestBody TaInvoiceDetail taInvoiceDetail) throws Exception {

        if (StringUtils.isEmpty(taInvoiceDetail.getInvoicePersonId())) {
            return ResponseBean.error("未选择报销人员");
        }

//        if (null == taInvoiceDetail.getItemList() || taInvoiceDetail.getItemList().size() < 1) {
//            return ResponseBean.error("未找到报销明细");
//        }

        TaInvoiceFill taInvoiceFill = iTaInvoiceFillService.getById(invoiceId);
        if (null == taInvoiceFill || taInvoiceFill.getStatus() == Constants.STATUS_DELETE) {
            return ResponseBean.error("报销单不存在");
        }

        if (getIsFinished(taInvoiceFill)) {
            return ResponseBean.error("报销单确认已结束");
        }

        TaPerson taPerson = currentPerson();
        taInvoiceDetail.setPersonId(taPerson.getPersonId());
        taInvoiceDetail.setInvoiceId(invoiceId);
        if (iTaInvoiceDetailService.createOrMergeInvoiceDetail(taInvoiceDetail)){
            return ResponseBean.success(taInvoiceDetail);
        } else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    private boolean getIsFinished(TaInvoiceFill taInvoiceFill) {
        return Constants.INVOICE_FINISHED == taInvoiceFill.getStatus() ||
                DateUtils.now().isAfter(taInvoiceFill.getEndDate());
    }

    @RequestMapping(value="/h5/invoice/{invoiceId}/detail",method= RequestMethod.GET)
    @ApiOperation(value="获取报销单信息", notes = "获取报销单信息", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getDetailByPerson(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                          @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                          @ApiParam("班级ID") @PathVariable(value ="invoiceId", required = false) String invoiceId,
                                          @ApiParam("人员ID") @PathVariable(value ="personId", required = false) String personId,
                                          @ApiParam("人员名称") @RequestParam(value ="personName", required = false) String personName) throws Exception {

        if (StringUtils.isEmpty(invoiceId) && StringUtils.isEmpty(personId)) {
            return ResponseBean.error("人员ID或者班级ID不能为空");
        }

        IPage<TaInvoiceDetail> page = new Page<>(pageNum, pageSize);
        return ResponseBean.success(iTaInvoiceDetailService.getPersonPage(page, invoiceId, personId, personName));
    }

    @RequestMapping(value="/h5/invoice/{invoiceId}/detail/info",method= RequestMethod.GET)
    @ApiOperation(value="获取报销单信息", notes = "获取报销单信息", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getDetailByPerson(@ApiParam("班级ID") @PathVariable(value ="invoiceId") String invoiceId,
                                          @ApiParam("人员ID") @RequestParam(value ="personId", required = false) String personId,
                                          @ApiParam("报销人员ID") @RequestParam(value ="invoicePersonId", required = false) String invoicePersonId) throws Exception{

        TaInvoiceFill taInvoiceFill = iTaInvoiceFillService.getById(invoiceId);
        if (null == taInvoiceFill || taInvoiceFill.getStatus() != Constants.STATUS_NORMAL) {
            return ResponseBean.error("报销单不存在");
        }

        if (StringUtils.isEmpty(personId) && StringUtils.isEmpty(invoicePersonId)) {
            return ResponseBean.error("人员ID不存在");
        }

        boolean isFinished = getIsFinished(taInvoiceFill);

        TaInvoiceDetail taInvoiceDetailTPl = null;
        if (!StringUtils.isEmpty(invoicePersonId)) {
            taInvoiceDetailTPl = iTaInvoiceDetailService.getTplByPerson(invoiceId, invoicePersonId);
        }

        TaInvoiceDetail taInvoiceDetail = null;
        if (!StringUtils.isEmpty(personId)) {
            taInvoiceDetail = iTaInvoiceDetailService.getByPerson(invoiceId, personId);
        }

        if (null == taInvoiceDetail && null == taInvoiceDetailTPl) {
            return ResponseBean.error("找不到报销数据信息");
        }

        // 获取明细
        if (StringUtils.isEmpty(invoicePersonId)) {
            invoicePersonId = taInvoiceDetail.getInvoicePersonId();
        }

        List<TaInvoiceItem> itemList = iTaInvoiceItemService.getListByPerson(invoiceId, invoicePersonId);
        if (null != taInvoiceDetailTPl) {
            taInvoiceDetailTPl.setItemList(itemList);
        }
        if (null != taInvoiceDetail) {
            taInvoiceDetail.setItemList(itemList);
        }


        Map<String, Object> resp = new HashMap<>();
        resp.put("detail", taInvoiceDetail);
        resp.put("tpl", taInvoiceDetailTPl);

        return ResponseBean.success(resp);
    }
}
