package com.njyunzhi.invoice.controller;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.metadata.Head;
import com.alibaba.excel.support.ExcelTypeEnum;
import com.alibaba.excel.write.merge.AbstractMergeStrategy;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.alibaba.excel.write.metadata.fill.FillConfig;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.njyunzhi.invoice.common.*;
import com.njyunzhi.invoice.common.excel.MergeRowStrategy;
import com.njyunzhi.invoice.entity.TaInvoiceDetail;
import com.njyunzhi.invoice.service.ITaInvoiceDetailService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Sheet;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.njyunzhi.invoice.service.ITaInvoiceFillService;
import com.njyunzhi.invoice.entity.TaInvoiceFill;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.InputStream;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
    * 报销主表 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2022-07-04
 */

@Api(tags = "报销主表")
@RestController
@RequestMapping("/")
public class TaInvoiceFillController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaInvoiceFillController.class);

    @Autowired
    public ITaInvoiceFillService iTaInvoiceFillService;

    @Autowired
    public ITaInvoiceDetailService iTaInvoiceDetailService;

    @Autowired
    FileUtil fileUtil;

    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/{client}/invoice",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taInvoiceFillList(@ApiParam("客户端") @PathVariable String client,
                                          @ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                          @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                          @ApiParam("班级名称") @RequestParam(value ="name", required = false) String name) throws Exception{
        IPage<TaInvoiceFill> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<TaInvoiceFill> queryWrapper = new QueryWrapper<>();
        queryWrapper.gt("status", Constants.STATUS_DELETE);
        queryWrapper.like(!StringUtils.isEmpty(name), "name", "%" + name + "%");
        queryWrapper.orderByDesc("create_date");

        // H5 端只能查看自己的报销单
        if ("h5".equals(client)) {
            queryWrapper.eq("person_id", currentPerson().getPersonId());
        }

        IPage<TaInvoiceFill> result = iTaInvoiceFillService.page(pg, queryWrapper);
        return ResponseBean.success(result);
    }

    @RequestMapping(value="/admin/invoice/{id}/export",method= RequestMethod.POST)
    @ApiOperation(value="导出", notes = "导出", httpMethod = "POST", response = ResponseBean.class)
    public void export(@ApiParam("班级ID") @PathVariable String id,
                               HttpServletResponse response) throws Exception {
        TaInvoiceFill invoiceFill = iTaInvoiceFillService.getById(id);
        if (null == invoiceFill) {
            throw new Exception("为找到班级信息");
        }

        // 获取填报信息
        List<TaInvoiceDetail> list = iTaInvoiceDetailService.getListByMainId(id);

        String tplPath = "excel/invoice_tpl.xlsx";  // 相对 resource 目录位置
        InputStream inputStream = fileUtil.getResourceFile(tplPath);
        String expFile = StringUtils.urlEncode(String.format("%s开票信息表.xlsx", invoiceFill.getName()));

        response.setContentType("application/vnd.ms-excel");
        response.setCharacterEncoding("utf-8");
        response.setHeader("Content-disposition", "attachment;filename="+expFile);
//        EasyExcel.write(response.getOutputStream(), TaInvoiceDetail.class)
//                .withTemplate(inputStream)
//                .excelType(ExcelTypeEnum.XLSX)
//                .sheet()
//                .doFill(list);

        List<Object> mergeDataList = list.stream().map(x -> x.getPersonName()).collect(Collectors.toList());
        ExcelWriter excelWriter = EasyExcel.write(response.getOutputStream(), TaInvoiceDetail.class).withTemplate(inputStream).build();
        WriteSheet writeSheet = EasyExcel.writerSheet().registerWriteHandler(new MergeRowStrategy(mergeDataList, 2, 0)).build();
        FillConfig fillConfig = FillConfig.builder().forceNewRow(true).build();

        excelWriter.fill(list, fillConfig, writeSheet);
        excelWriter.finish();
    }

    /**
     * 保存对象
     * @param taInvoiceFill 实体对象
     * @return
     */
    @RequestMapping(value="/admin/invoice",method= RequestMethod.POST)
    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean taInvoiceFillAdd(@ApiParam("保存内容") @RequestBody TaInvoiceFill taInvoiceFill) throws Exception{

        if (null == taInvoiceFill.getStatus()) {
            taInvoiceFill.setStatus(Constants.STATUS_READY);
        }

        if (iTaInvoiceFillService.save(taInvoiceFill)){
            return ResponseBean.success(taInvoiceFill);
        }else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/invoice/{id}", method= RequestMethod.DELETE)
    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
    public ResponseBean taInvoiceFillDelete(@ApiParam("对象ID") @PathVariable String id) throws Exception{
        if(iTaInvoiceFillService.removeLogicById(id)){
            return ResponseBean.success("success");
        }else {
            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param taInvoiceFill 实体对象
     * @return
     */
    @RequestMapping(value="/admin/invoice/{id}",method= RequestMethod.PUT)
    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean taInvoiceFillUpdate(@ApiParam("对象ID") @PathVariable String id,
                                        @ApiParam("更新内容") @RequestBody TaInvoiceFill taInvoiceFill) throws Exception{

        taInvoiceFill.setInvoiceId(id);
        if (iTaInvoiceFillService.updateById(taInvoiceFill)){
            return ResponseBean.success(iTaInvoiceFillService.getById(id));
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/invoice/{id}",method= RequestMethod.GET)
    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taInvoiceFillGet(@ApiParam("对象ID") @PathVariable String id) throws Exception{
        return ResponseBean.success(iTaInvoiceFillService.getById(id));
    }


    @RequestMapping(value="/h5/invoice/{invoiceId}",method= RequestMethod.GET)
    @ApiOperation(value="获取报销单", notes = "获取报销单", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getInvoice(@ApiParam("班级ID") @PathVariable(value ="invoiceId") String invoiceId) throws Exception{
        TaInvoiceFill taInvoiceFill = iTaInvoiceFillService.getById(invoiceId);
        if (null == taInvoiceFill || taInvoiceFill.getStatus() != Constants.STATUS_NORMAL) {
            return ResponseBean.error("报销单不存在");
        }

        return ResponseBean.success(taInvoiceFill);
    }
}
