package com.njyunzhi.invoice.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.njyunzhi.invoice.common.BaseController;
import com.njyunzhi.invoice.common.Constants;
import com.njyunzhi.invoice.common.ResponseBean;
import com.njyunzhi.invoice.common.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.njyunzhi.invoice.service.ITaInvoiceOrgService;
import com.njyunzhi.invoice.entity.TaInvoiceOrg;
import org.springframework.web.bind.annotation.RestController;

/**
 * <p>
    * 报销单位 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2022-07-04
 */

@Api(tags = "报销单位")
@RestController
@RequestMapping("/")
public class TaInvoiceOrgController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaInvoiceOrgController.class);

    @Autowired
    public ITaInvoiceOrgService iTaInvoiceOrgService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/{client}/invoice/{invoiceId}/org",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taInvoiceOrgList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                         @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                         @ApiParam("班级ID") @PathVariable(value ="invoiceId") String invoiceId,
                                         @ApiParam("单位名称") @RequestParam(value ="name", required = false) String name) throws Exception{

        IPage<TaInvoiceOrg> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<TaInvoiceOrg> queryWrapper = new QueryWrapper<>();
        queryWrapper.gt("status", Constants.STATUS_DELETE);
        queryWrapper.like(!StringUtils.isEmpty(name),"name", "%" + name + "%");
        queryWrapper.eq("invoice_id", invoiceId);
        queryWrapper.orderByDesc("create_date");

        IPage<TaInvoiceOrg> result = iTaInvoiceOrgService.page(pg, queryWrapper);
        return ResponseBean.success(result);
    }
}
