package com.njyunzhi.invoice.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.njyunzhi.invoice.common.BaseController;
import com.njyunzhi.invoice.common.Constants;
import com.njyunzhi.invoice.common.ResponseBean;
import com.njyunzhi.invoice.common.StringUtils;
import com.njyunzhi.invoice.entity.TaInvoiceOrg;
import com.njyunzhi.invoice.entity.TaPerson;
import com.njyunzhi.invoice.service.ITaInvoiceOrgService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.njyunzhi.invoice.service.ITaInvoicePersonService;
import com.njyunzhi.invoice.entity.TaInvoicePerson;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

/**
 * <p>
    * 报销人员 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2022-07-04
 */

@Api(tags = "报销人员")
@RestController
@RequestMapping("/")
public class TaInvoicePersonController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaInvoicePersonController.class);

    @Autowired
    public ITaInvoicePersonService iTaInvoicePersonService;

    @Autowired
    public ITaInvoiceOrgService iTaInvoiceOrgService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/{client}/invoice/{invoiceId}/person",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taInvoicePersonList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                            @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                            @ApiParam("班级ID") @PathVariable(value ="invoiceId") String invoiceId,
                                            @ApiParam("所属单位") @RequestParam(value ="invoiceOrgId", required = false) String invoiceOrgId,
                                            @ApiParam("人员名称") @RequestParam(value ="name", required = false) String name) throws Exception {
        IPage<TaInvoicePerson> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<TaInvoicePerson> queryWrapper = new QueryWrapper<>();
        queryWrapper.gt("status", Constants.STATUS_DELETE);
        queryWrapper.eq("invoice_id", invoiceId);
        queryWrapper.like(!StringUtils.isEmpty(name),"name", "%" + name + "%");
        queryWrapper.eq(!StringUtils.isEmpty(invoiceOrgId), "invoice_org_id", invoiceOrgId);
        queryWrapper.orderByDesc("create_date");
        IPage<TaInvoicePerson> result = iTaInvoicePersonService.page(pg,queryWrapper);
        return ResponseBean.success(result);
    }

    /**
     * 保存对象
     * @param taInvoicePerson 实体对象
     * @return
     */
    @RequestMapping(value="/admin/invoice/{invoiceId}/person",method= RequestMethod.POST)
    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean taInvoicePersonAdd(@ApiParam("班级ID") @PathVariable(value ="invoiceId") String invoiceId,
                                           @ApiParam("保存内容") @RequestBody TaInvoicePerson taInvoicePerson) throws Exception {
        if (StringUtils.isEmpty(taInvoicePerson.getOrgName())) {
            return ResponseBean.error("未找到单位信息");
        }

        taInvoicePerson.setInvoiceId(invoiceId);
        if (iTaInvoicePersonService.createNew(taInvoicePerson)){
            return ResponseBean.success(taInvoicePerson);
        } else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 批量导入
     * @return
     */
    @RequestMapping(value="/admin/invoice/{invoiceId}/person/import",method= RequestMethod.POST)
    @ApiOperation(value="批量导入", notes = "批量导入", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean upload(@ApiParam("班级ID") @PathVariable(value = "invoiceId") String invoiceId,
                               @ApiParam("保存内容") @RequestParam(value = "file") MultipartFile file) throws Exception {
        iTaInvoicePersonService.batchUpload(invoiceId, file);
        return ResponseBean.success("ok");
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/invoice/{invoiceId}/person/{id}", method= RequestMethod.DELETE)
    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
    public ResponseBean taInvoicePersonDelete(@ApiParam("对象ID") @PathVariable String id) throws Exception{
        if (iTaInvoicePersonService.removeLogicById(id)) {
            return ResponseBean.success("success");
        } else {
            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }


//    /**
//     * 根据id删除对象
//     * @param id  实体ID
//     */
//    @RequestMapping(value="/h5/invoice/{invoiceId}/person/{id}/bind", method= RequestMethod.DELETE)
//    @ApiOperation(value="绑定人员", notes = "绑定人员", httpMethod = "DELETE", response = ResponseBean.class)
//    public ResponseBean bind(@ApiParam("班级ID") @PathVariable String invoiceId,
//                             @ApiParam("人员ID") @PathVariable String id) throws Exception {
//
//        TaInvoicePerson taInvoicePerson = iTaInvoicePersonService.getByPerson(invoiceId, id);
//        TaPerson taPerson = currentPerson();
//
//        if(iTaInvoicePersonService.bind(taInvoicePerson, taPerson)){
//            return ResponseBean.success("success");
//        }else {
//            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
//        }
//    }
}
