package com.njyunzhi.invoice.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.njyunzhi.invoice.common.Constants;
import com.njyunzhi.invoice.common.DateUtils;
import com.njyunzhi.invoice.entity.TaInvoiceDetail;
import com.njyunzhi.invoice.entity.TaInvoiceDetailTpl;
import com.njyunzhi.invoice.entity.TaInvoiceItem;
import com.njyunzhi.invoice.entity.TaInvoiceItemTpl;
import com.njyunzhi.invoice.mapper.TaInvoiceDetailMapper;
import com.njyunzhi.invoice.mapper.TaInvoiceDetailTplMapper;
import com.njyunzhi.invoice.mapper.TaInvoiceItemMapper;
import com.njyunzhi.invoice.mapper.TaInvoiceItemTplMapper;
import com.njyunzhi.invoice.service.ITaInvoiceDetailService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 报销明细 服务实现类
 * </p>
 *
 * @author yansen
 * @since 2022-07-04
 */
@Service
public class TaInvoiceDetailServiceImpl extends ServiceImpl<TaInvoiceDetailMapper, TaInvoiceDetail> implements ITaInvoiceDetailService {

    @Autowired
    TaInvoiceDetailTplMapper taInvoiceDetailTplMapper;

    @Autowired
    TaInvoiceItemMapper taInvoiceItemMapper;

    @Autowired
    TaInvoiceItemTplMapper taInvoiceItemTplMapper;

    @Override
    public boolean createBatch(String invoiceId) throws Exception {
        // 先删除原有数据
        Map<String, Object> con = new HashMap<String, Object>(){{
            put("invoice_id", invoiceId);
        }};
        taInvoiceItemMapper.deleteByMap(con);

        return taInvoiceItemMapper.createBatch(invoiceId) > 0;
    }

    @Override
    public TaInvoiceDetail getByPerson(String invoiceId, String personId) {
        return baseMapper.getByPerson(invoiceId, personId);
    }

    @Override
    public List<TaInvoiceDetail> getListByMainId(String invoiceId) {
        return baseMapper.getListByMainId(invoiceId);
    }

    @Override
    public IPage<TaInvoiceDetail> getTplPageBy(IPage<TaInvoiceDetail> pg, String invoiceId) {
        return baseMapper.getTplPageBy(pg, invoiceId);
    }

    @Override
    public TaInvoiceDetail getTplByPerson(String invoiceId, String invoicePersonId) {
        return baseMapper.getTplByPerson(invoiceId, invoicePersonId);
    }

    @Override
    public IPage<TaInvoiceDetail> getPersonPage(IPage<TaInvoiceDetail> page, String invoiceId, String personId, String personName) {
        return baseMapper.getPersonPage(page, invoiceId, personId, personName);
    }

    @Override
    public boolean createOrMergeInvoiceDetail(TaInvoiceDetail taInvoiceDetail) {
//        // 更新报销项目
//        for (TaInvoiceItem item : taInvoiceDetail.getItemList()) {
//            taInvoiceItemMapper.updateById(item);
//        }

        // 查询原始记录
        TaInvoiceDetail origin = baseMapper.getByPerson(taInvoiceDetail.getInvoiceId(), taInvoiceDetail.getPersonId());

        // 如果不是新生成, 二次编辑
        if (null != origin) {
            origin.setInvoicePersonId(taInvoiceDetail.getInvoicePersonId());
            origin.setPersonName(taInvoiceDetail.getPersonName());
            origin.setInvoiceOrgId(taInvoiceDetail.getInvoiceOrgId());
            origin.setInvoiceOrgName(taInvoiceDetail.getInvoiceOrgName());
            origin.setOrgId(taInvoiceDetail.getOrgId());
            origin.setOrgName(taInvoiceDetail.getOrgName());
            origin.setTaxNo(taInvoiceDetail.getTaxNo());
            origin.setAddress(taInvoiceDetail.getAddress());
            origin.setPhone(taInvoiceDetail.getPhone());
            origin.setBankId(taInvoiceDetail.getBankId());
            origin.setBankName(taInvoiceDetail.getBankName());
            origin.setCardNo(taInvoiceDetail.getCardNo());
            origin.setPersonId(taInvoiceDetail.getPersonId());
            origin.setMergeRemark(taInvoiceDetail.getMergeRemark());
            origin.setStayRemark(taInvoiceDetail.getStayRemark());

            return updateById(origin);
        }

        // 否则就是全新生成
        return save(taInvoiceDetail);
    }

    @Override
    public boolean createOrUpdateItem(TaInvoiceItem taInvoiceItem) throws Exception {
        TaInvoiceItem origin = taInvoiceItemMapper.getByPersonAndName(taInvoiceItem.getInvoiceId(), taInvoiceItem.getInvoicePersonId(), taInvoiceItem.getName());
        if (origin != null) {
            origin.setCharge(taInvoiceItem.getCharge());
            return taInvoiceItemMapper.updateById(origin) > 0;
        }

        taInvoiceItem.setStatus(Constants.STATUS_NORMAL);
        taInvoiceItem.setCreateDate(DateUtils.now());

        return taInvoiceItemMapper.insert(taInvoiceItem) > 0;
    }

    @Override
    public boolean deleteItem(String invoiceId, Integer itemId) {
        return taInvoiceItemMapper.deleteById(itemId) > 0;
    }

    @Override
    public IPage<TaInvoiceDetail> getPageBy(IPage<TaInvoiceDetail> pg, String invoiceId, String orgName, String personName) {
        return baseMapper.getPageBy(pg, invoiceId, orgName, personName);
    }
}
